Unit GrIP;

Interface

Type
  GRIP_SLOT     = Byte;
  GRIP_CLASS    = Byte;
  GRIP_INDEX    = Byte;
  GRIP_VALUE    = Word;
  GRIP_BITFIELD = LongInt;
  GRIP_BOOL     = Word;
  GRIP_STRING   = Pointer;
  TGRIP_BUF     = Pointer;
  GRIP_BUF_C    = ^Byte;
  GRIP_BUF_S    = ^Word;
  GRIP_BUF_L    = ^LongInt;

{ API Call Thunk }
  TGRIP_THUNK = Array[0..13] Of Byte;

Const
  _32BIT_OPERAND = $66;
  _32BIT_ADDRESS = $67;

  { Standard Classes }

  GRIP_CLASS_BUTTON   = 1;
  GRIP_CLASS_AXIS     = 2;
  GRIP_CLASS_POV_HAT  = 3;
  GRIP_CLASS_VELOCITY = 4;
  GRIP_CLASS_THROTTLE = 5;
  GRIP_CLASS_ANALOG   = 6;

  { Refresh Flags }

  GRIP_REFRESH_COMPLETE   = 0; { Default }
  GRIP_REFRESH_PARTIAL    = 2;
  GRIP_REFRESH_TRANSMIT   = 0; { Default }
  GRIP_REFRESH_NOTRANSMIT = 4;

{ Standard String Constants for Loadable Library }

Var
  Grip_Thunk   : TGRIP_THUNK;
  GrIP_Loaded  : Boolean;
  GrIP_Image   : Pointer;
  GrIP_ImgSize : Word;

Const
{ Standard String Constants for Loadable Library }
  GrLibName    : String = 'GRIP.GLL';
  GrLibEnv     : String = 'GRIP';
  GrLibDir     : String = 'C:\GRIP\';
  GRIP_Thunked : Boolean = False;

{ 3.1 System API Calls }

Function  GrInitialize: GRIP_BOOL;
Procedure GrShutdown;
Function  GrRefresh(Flags: GRIP_BITFIELD): GRIP_BITFIELD;

{ 3.2 Configuration API Calls }

Function  GrGetSlotMap: GRIP_BITFIELD;
Function  GrGetClassMap(S: GRIP_SLOT): GRIP_BITFIELD;
Function  GrGetOEMClassMap(S: GRIP_SLOT): GRIP_BITFIELD;
Function  GrGetMaxIndex(S: GRIP_SLOT; C: GRIP_CLASS): GRIP_INDEX;
Function  GrGetMaxValue(S: GRIP_SLOT; C: GRIP_CLASS): GRIP_VALUE;

{ 3.3 Data API Calls }

Function  GrGetValue(S: GRIP_SLOT; C: GRIP_CLASS; I: GRIP_INDEX): GRIP_VALUE;
Function  GrGetPackedValues(S: GRIP_SLOT; C: GRIP_CLASS; IStart: GRIP_INDEX; IEnd: GRIP_INDEX): GRIP_BITFIELD;
Procedure GrSetValue(S: GRIP_SLOT; C: GRIP_CLASS; I: GRIP_INDEX; V: GRIP_VALUE);

{ 3.4 OEM Information API Calls }

Procedure GrGetVendorName(S: GRIP_SLOT; Name: GRIP_STRING);
Function  GrGetProductName(S: GRIP_SLOT; Name: GRIP_STRING): GRIP_VALUE;
Procedure GrGetControlName(S: GRIP_SLOT; C: GRIP_CLASS; I: GRIP_INDEX; Name: GRIP_STRING);
Function  GrGetCaps(S: GRIP_SLOT; C: GRIP_CLASS; I: GRIP_INDEX): GRIP_BITFIELD;

{ 3.5 Library Management Calls }

Function  GrLink(Image: TGRIP_BUF; Size: GRIP_VALUE): Boolean;
Function  GrUnlink: Boolean;

Function  Gr__Link: GRIP_BOOL;
Procedure Gr__Unlink;

{ Diagnostic Information Calls }

Function  GrGetSWVer: GRIP_VALUE;
Function  GrGetHWVer: GRIP_VALUE;
Function  GrGetDiagCnt: GRIP_VALUE;
Function  GrGetDiagReg(Reg: GRIP_INDEX): LongInt;

Implementation

Uses Dos;

{ Global Thunking Data }

Var
  GRIP_CS        : Word;
  GRIP_DS        : Word;
  GRIP_Buf       : Pointer;
  GRIP_Size      : Word;
  Gr__LinkResult : Word;
  OldExitProc    : Pointer;

(*
#define GRIP__PACK_EDX                      \
  "shl edx, 16                            " \
  "mov dx, ax                             "
#define GRIP__UNPACK_EAX                    \
  "mov edx, eax                           " \
  "shr edx, 16                            "
#define GRIP__UNPACK_STRING                 \
  "mov es:[di][00h], ebx                  " \
  "mov es:[di][04h], ecx                  " \
  "mov es:[di][08h], edx                  " \
  "mov es:[di][0Ch], esi                  "
#define GRIP__REG_EDX          dx ax
#define GRIP__REG_EAX          dx ax
#define GRIP__REG_STRING       es di
#define GRIP__MOD_STRING       bx cx dx si
#define GRIP__CALL                          \
  "push ds                                " \
  "push seg GRIP_Thunk                    " \
  "pop ds                                 " \
  "call far [GRIP_Thunk+0]                " \
  "pop ds                                 "
*)

(*
#pragma aux GrInitialize =                  \
  "mov ax, 84A1h                          " \
  GRIP__CALL                                \
  value [ax];

#pragma aux GrShutdown =                    \
  "mov ax, 84A2h                          " \
  GRIP__CALL                                \
  modify [ax];

#pragma aux GrRefresh =                     \
  GRIP__PACK_EDX                            \
  "mov ax, 84A0h                          " \
  GRIP__CALL                                \
  GRIP__UNPACK_EAX                          \
  modify [ax]                               \
  parm [GRIP__REG_EDX]                      \
  value [GRIP__REG_EAX];

#pragma aux GrGetSlotMap =                  \
  "mov ax, 84B0h                          " \
  GRIP__CALL                                \
  GRIP__UNPACK_EAX                          \
  value [GRIP__REG_EAX];

#pragma aux GrGetClassMap =                 \
  "mov ax, 84B1h                          " \
  GRIP__CALL                                \
  GRIP__UNPACK_EAX                          \
  parm [bh]                                 \
  value [GRIP__REG_EAX];

#pragma aux GrGetOEMClassMap =              \
  "mov ax, 84B2h                          " \
  GRIP__CALL                                \
  GRIP__UNPACK_EAX                          \
  parm [bh]                                 \
  value [GRIP__REG_EAX];

#pragma aux GrGetMaxIndex =                 \
  "mov ax, 84B3h                          " \
  GRIP__CALL                                \
  parm [bh] [bl]                            \
  value [al];

#pragma aux GrGetMaxValue =                 \
  "mov ax, 84B4h                          " \
  GRIP__CALL                                \
  parm [bh] [bl]                            \
  value [ax];

#pragma aux GrGetValue =                    \
  "mov ax, 84C0h                          " \
  GRIP__CALL                                \
  parm [bh] [bl] [ch]                       \
  value [ax];

#pragma aux GrGetPackedValues =             \
  "mov ax, 84C1h                          " \
  GRIP__CALL                                \
  GRIP__UNPACK_EAX                          \
  parm [bh] [bl] [ch] [cl]                  \
  value [GRIP__REG_EAX];

#pragma aux GrSetValue =                    \
  "mov ax, 84C2h                          " \
  GRIP__CALL                                \
  parm [bh] [bl] [ch] [dx];

#pragma aux GrGetVendorName =               \
  "mov ax, 84D0h                          " \
  GRIP__CALL                                \
  GRIP__UNPACK_STRING                       \
  modify [GRIP__MOD_STRING ax]              \
  parm [bh] [GRIP__REG_STRING];

#pragma aux GrGetProductName =              \
  "mov ax, 84D1h                          " \
  GRIP__CALL                                \
  GRIP__UNPACK_STRING                       \
  modify [GRIP__MOD_STRING]                 \
  parm [bh] [GRIP__REG_STRING]              \
  value [ax];

#pragma aux GrGetControlName =              \
  "mov ax, 84D2h                          " \
  GRIP__CALL                                \
  GRIP__UNPACK_STRING                       \
  modify [GRIP__MOD_STRING ax]              \
  parm [bh] [bl] [ch] [GRIP__REG_STRING];

#pragma aux GrGetCaps =                     \
  "xor eax, eax                           " \
  GRIP__UNPACK_EAX                          \
  parm [bh] [bl] [ch]                       \
  value [GRIP__REG_EAX];

#pragma aux Gr__Link =                      \
  "mov ax, 8490h                          " \
  GRIP__CALL                                \
  value [ax];

#pragma aux Gr__Unlink =                    \
  "mov ax, 8491h                          " \
  GRIP__CALL                                \
  modify [ax];

#pragma aux GrGetSWVer =                    \
  "mov ax, 84F0h                          " \
  GRIP__CALL                                \
  value [ax];

#pragma aux GrGetHWVer =                    \
  "mov ax, 84F1h                          " \
  GRIP__CALL                                \
  value [ax];

#pragma aux GrGetDiagCnt =                  \
  "mov ax, 84F2h                          " \
  GRIP__CALL                                \
  value [ax];

#pragma aux GrGetDiagReg =                  \
  "mov ax, 84F3h                          " \
  GRIP__CALL                                \
  GRIP__UNPACK_EAX                          \
  parm [bh]                                 \
  value [GRIP__REG_EAX];
*)

Function GrInitialize: GRIP_BOOL; Assembler;
Asm
  mov   ax,84A1h
  push  ds                        { GRIP__CALL }
  push  seg GRIP_Thunk
  pop   ds
  call  DWORD PTR GRIP_Thunk { far [GRIP_Thunk+0] }
  pop   ds
End; { grInitialize }

Procedure GrShutdown; Assembler;
Asm
  mov   ax,84A2h
  push  ds                        { GRIP__CALL }
  push  seg GRIP_Thunk
  pop   ds
  call  DWORD PTR GRIP_Thunk { far [GRIP_Thunk+0] }
  pop   ds
End; { GrShutDown }

Function GrRefresh(Flags: GRIP_BITFIELD): GRIP_BITFIELD;
Var L: LongInt;
Begin
  Asm
    DB    _32BIT_OPERAND
    MOV   DX,WORD PTR Flags
    mov   ax,84A0h
    push  ds                      { GRIP__CALL }
    push  seg GRIP_Thunk
    pop   ds
    call  DWORD PTR GRIP_Thunk { far [GRIP_Thunk+0] }
    pop   ds
    DB    _32BIT_OPERAND          { GRIP__UNPACK_EAX }
    MOV   WORD PTR L,AX
  End; { Asm }
  GrRefresh := L;
End; { GrRefresh }

Function GrGetSlotMap: GRIP_BITFIELD; Assembler;
Asm
  mov   ax,84B0h
  push  ds                      { GRIP__CALL }
  push  seg GRIP_Thunk
  pop   ds
  call  DWORD PTR GRIP_Thunk { far [GRIP_Thunk+0] }
  pop   ds
  DB    _32BIT_OPERAND          { GRIP__UNPACK_EAX }
  mov   dx,ax
  DB    _32BIT_OPERAND
  shr   dx,16
End; { GrGetSlotMap }

Function GrGetClassMap(S: GRIP_SLOT): GRIP_BITFIELD;
Var L: LongInt;
Begin
  Asm
    mov   ax,84B1h
    MOV   BH,BYTE PTR S
    push  ds                      { GRIP__CALL }
    push  seg GRIP_Thunk
    pop   ds
    call  DWORD PTR GRIP_Thunk { far [GRIP_Thunk+0] }
    pop   ds
    DB    _32BIT_OPERAND          { GRIP__UNPACK_EAX }
    MOV   WORD PTR L,AX
  End; { Asm }
  GrGetClassMap := L;
End; { GrGetClassMap }

Function GrGetOEMClassMap(S: GRIP_SLOT): GRIP_BITFIELD;
Var L: LongInt;
Begin
  Asm
    mov   ax,84B2h
    MOV   BH,BYTE PTR S
    push  ds                      { GRIP__CALL }
    push  seg GRIP_Thunk
    pop   ds
    call  DWORD PTR GRIP_Thunk { far [GRIP_Thunk+0] }
    pop   ds
    DB    _32BIT_OPERAND          { GRIP__UNPACK_EAX }
    MOV   WORD PTR L,AX
  End; { Asm }
  GrGetOEMClassMap := L;
End; { GrGetOEMClassMap }

Function GrGetMaxIndex(S: GRIP_SLOT; C: GRIP_CLASS): GRIP_INDEX;
Var B: Byte;
Begin
  Asm
    mov   ax,84B3h
    MOV   BH,BYTE PTR S
    MOV   BL,BYTE PTR C
    push  ds                      { GRIP__CALL }
    push  seg GRIP_Thunk
    pop   ds
    call  DWORD PTR GRIP_Thunk { far [GRIP_Thunk+0] }
    pop   ds
    MOV   BYTE PTR B,AL
  End; { Asm }
  GrGetMaxIndex := B;
End; { GrGetMaxIndex }

Function GrGetMaxValue(S: GRIP_SLOT; C: GRIP_CLASS): GRIP_VALUE;
Var W: Word;
Begin
  Asm
    mov   ax,84B4h
    MOV   BH,BYTE PTR S
    MOV   BL,BYTE PTR C
    push  ds                      { GRIP__CALL }
    push  seg GRIP_Thunk
    pop   ds
    call  DWORD PTR GRIP_Thunk { far [GRIP_Thunk+0] }
    pop   ds
    MOV   WORD PTR W,AX
  End; { Asm }
  GrGetMaxValue := W;
End; { GrGetMaxValue }

Function GrGetValue(S: GRIP_SLOT; C: GRIP_CLASS; I: GRIP_INDEX): GRIP_VALUE;
Var W: Word;
Begin
  Asm
    mov   ax,84C0h
    MOV   BH,BYTE PTR S
    MOV   BL,BYTE PTR C
    MOV   CH,BYTE PTR I
    push  ds                      { GRIP__CALL }
    push  seg GRIP_Thunk
    pop   ds
    call  DWORD PTR GRIP_Thunk { far [GRIP_Thunk+0] }
    pop   ds
    MOV   WORD PTR W,AX
  End; { Asm }
  GrGetValue := W;
End; { GrGetValue }

Function GrGetPackedValues(S: GRIP_SLOT; C: GRIP_CLASS; IStart: GRIP_INDEX; IEnd: GRIP_INDEX): GRIP_BITFIELD;
Var L: LongInt;
Begin
  Asm
    mov   ax,84C1h
    MOV   BH,BYTE PTR S
    MOV   BL,BYTE PTR C
    MOV   CH,BYTE PTR IStart
    MOV   CL,BYTE PTR IEnd
    push  ds                      { GRIP__CALL }
    push  seg GRIP_Thunk
    pop   ds
    call  DWORD PTR GRIP_Thunk { far [GRIP_Thunk+0] }
    pop   ds
    DB    _32BIT_OPERAND          { GRIP__UNPACK_EAX }
    MOV   WORD PTR L,AX
  End; { Asm }
  GrGetPackedValues := L;
End; { GrGetPackedValues }

Procedure GrSetValue(S: GRIP_SLOT; C: GRIP_CLASS; I: GRIP_INDEX; V: GRIP_VALUE);
Begin
  Asm
    mov   ax,84C2h
    MOV   BH,BYTE PTR S
    MOV   BL,BYTE PTR C
    MOV   CH,BYTE PTR I
    MOV   DX,WORD PTR V
    push  ds                      { GRIP__CALL }
    push  seg GRIP_Thunk
    pop   ds
    call  DWORD PTR GRIP_Thunk { far [GRIP_Thunk+0] }
    pop   ds
  End; { Asm }
End; { GrSetValue }

Procedure GrGetVendorName(S: GRIP_SLOT; Name: GRIP_STRING);
Begin
  Asm
    mov   ax,84D0h
    MOV   BH,BYTE PTR S
    LES   DI,DWORD PTR Name
    push  ds                      { GRIP__CALL }
    push  seg GRIP_Thunk
    pop   ds
    call  DWORD PTR GRIP_Thunk { far [GRIP_Thunk+0] }
    pop   ds
    DB    _32BIT_OPERAND
    mov   word ptr es:[di+00h],bx { GRIP__UNPACK_STRING }
    DB    _32BIT_OPERAND
    mov   word ptr es:[di+04h],cx
    DB    _32BIT_OPERAND
    mov   word ptr es:[di+08h],dx
    DB    _32BIT_OPERAND
    mov   word ptr es:[di+0Ch],si
  End; { Asm }
End; { GrGetVendorName }

Function GrGetProductName(S: GRIP_SLOT; Name: GRIP_STRING): GRIP_VALUE;
Var W: Word;
Begin
  Asm
    mov   ax,84D1h
    MOV   BH,BYTE PTR S
    LES   DI,DWORD PTR Name
    push  ds                      { GRIP__CALL }
    push  seg GRIP_Thunk
    pop   ds
    call  DWORD PTR GRIP_Thunk { far [GRIP_Thunk+0] }
    pop   ds
    DB    _32BIT_OPERAND
    mov   word ptr es:[di+00h],bx { GRIP__UNPACK_STRING }
    DB    _32BIT_OPERAND
    mov   word ptr es:[di+04h],cx
    DB    _32BIT_OPERAND
    mov   word ptr es:[di+08h],dx
    DB    _32BIT_OPERAND
    mov   word ptr es:[di+0Ch],si
    MOV   WORD PTR W,AX
  End; { Asm }
  GrGetProductName := W;
End; { GrGetProductName }

Procedure GrGetControlName(S: GRIP_SLOT; C: GRIP_CLASS; I: GRIP_INDEX; Name: GRIP_STRING);
Begin
  Asm
    mov   ax,84D2h
    MOV   BH,BYTE PTR S
    MOV   BL,BYTE PTR C
    MOV   CH,BYTE PTR I
    LES   DI,DWORD PTR Name
    push  ds                      { GRIP__CALL }
    push  seg GRIP_Thunk
    pop   ds
    call  DWORD PTR GRIP_Thunk { far [GRIP_Thunk+0] }
    pop   ds
    DB    _32BIT_OPERAND
    mov   word ptr es:[di+00h],bx { GRIP__UNPACK_STRING }
    DB    _32BIT_OPERAND
    mov   word ptr es:[di+04h],cx
    DB    _32BIT_OPERAND
    mov   word ptr es:[di+08h],dx
    DB    _32BIT_OPERAND
    mov   word ptr es:[di+0Ch],si
  End; { Asm }
End; { GrGetControlName }

Function GrGetCaps(S: GRIP_SLOT; C: GRIP_CLASS; I: GRIP_INDEX): GRIP_BITFIELD;
Var L: LongInt;
Begin
  Asm
    MOV   BH,BYTE PTR S
    MOV   BL,BYTE PTR C
    MOV   CH,BYTE PTR I
    DB    _32BIT_OPERAND
    xor   ax,ax
    DB    _32BIT_OPERAND          { GRIP__UNPACK_EAX }
    MOV   WORD PTR L,AX
  End; { Asm }
  GrGetCaps := L;
End; { GrGetControlName }

Function Gr__Link: GRIP_BOOL; Assembler;
Asm
  mov   ax,8490h
  push  ds                        { GRIP__CALL }
  push  seg GRIP_Thunk
  pop   ds
  call  DWORD PTR GRIP_Thunk { far [GRIP_Thunk+0] }
  pop   ds
  MOV   WORD PTR Gr__LinkResult,AX
End; { Gr__Link }

Procedure Gr__Unlink; Assembler;
Asm
  mov   ax,8491h
  push  ds                        { GRIP__CALL }
  push  seg GRIP_Thunk
  pop   ds
  call  DWORD PTR GRIP_Thunk { far [GRIP_Thunk+0] }
  pop   ds
End; { Gr__Unlink }

Function GrGetSWVer: GRIP_VALUE; Assembler;
Asm
  mov   ax,84F0h
  push  ds                        { GRIP__CALL }
  push  seg GRIP_Thunk
  pop   ds
  call  DWORD PTR GRIP_Thunk { far [GRIP_Thunk+0] }
  pop   ds
End; { GrGetSWVer }

Function GrGetHWVer: GRIP_VALUE; Assembler;
Asm
  mov   ax,84F1h
  push  ds                        { GRIP__CALL }
  push  seg GRIP_Thunk
  pop   ds
  call  DWORD PTR GRIP_Thunk { far [GRIP_Thunk+0] }
  pop   ds
End; { GrGetHWVer }

Function GrGetDiagCnt: GRIP_VALUE; Assembler;
Asm
  mov   ax,84F2h
  push  ds                        { GRIP__CALL }
  push  seg GRIP_Thunk
  pop   ds
  call  DWORD PTR GRIP_Thunk { far [GRIP_Thunk+0] }
  pop   ds
End; { GrGetDiagCnt }

Function GrGetDiagReg(Reg: GRIP_INDEX): LongInt;
Var L: LongInt;
Begin
  Asm
    mov   ax,84F3h
    MOV   BH,BYTE PTR Reg
    push  ds                      { GRIP__CALL }
    push  seg GRIP_Thunk
    pop   ds
    call  DWORD PTR GRIP_Thunk { far [GRIP_Thunk+0] }
    pop   ds
    DB    _32BIT_OPERAND          { GRIP__UNPACK_EAX }
    MOV   WORD PTR L,AX
  End; { Asm }
  GrGetDiagReg := L;
End; { GrGetDiagReg }


Function GrLink(Image: TGRIP_BUF; Size: GRIP_VALUE): Boolean;
Type
  ArPtr = ^Ar;
  Ar    = Array[0..32767] Of Byte;

Var
  Img_Off   : Word;        { Offset of binary image   }
  Img_Size  : Word;        { Size of binary image     }
  Img_Check : Byte;        { Checksum of binary image }
  Header    : ArPtr;       { Image header             }
  Core      : ArPtr;       { Image core               }
  Memory    : ArPtr;       { Image memory             }
  I         : Integer;

Begin

  { Check for duplicate GrLink calls }

  If GRIP_Thunked Then
  Begin
    GrLink := False;
    Exit;
  End;

  { Sanity check the inputs }

  If (Image = Nil) Or (Size < 16) Then
  Begin
    GrLink := False;
    Exit;
  End;

  { Check the image header for the proper signature bytes }

  Header := ArPtr(Image);
  If MemW[Seg(Header^):Ofs(Header^)] <> $4244 Then
  Begin
    GrLink := False;
    Exit;
  End;

  { Extract the image offsets from the header }

  Img_Off := ( (Header^[2] And $0F) Shl 12 ) Or
             ( (Header^[3] And $0F) Shl  8 ) Or
             ( (Header^[4] And $0F) Shl  4 ) Or
             ( (Header^[5] And $0F) );

  Img_Size := ( (Header^[6] And $0F) Shl 12 ) Or
              ( (Header^[7] And $0F) Shl  8 ) Or
              ( (Header^[8] And $0F) Shl  4 ) Or
              ( (Header^[9] And $0F) );

  Img_Check := ( (Header^[10] And $0F) Shl 4 ) Or
               ( (Header^[11] And $0F) );

  { Compute the start of image core }

  If Size < ((Img_Off + Img_Size) Shl 4) Then
  Begin
    GrLink := False;
    Exit;
  End;

  Core := ArPtr(LongInt(Header) + (Img_Off Shl 4));

  { Verify the checksum }

  For I := 0 To (Img_Size Shl 4) - 1 Do
   Img_Check := Img_Check Xor Core^[I];
  If Img_Check <> 0 Then
  Begin
    GrLink := False;
    Exit;
  End;

  { Allocate memory for the core }

  GRIP_Size := (Img_Size + 1) Shl 4;
  GetMem(GRIP_Buf,GRIP_Size);

  If GRIP_Buf = Nil Then
  Begin
    GrLink := False;
    Exit;
  End;

{
    GRIP_DS = (unsigned short)
             ( (
                 ((((unsigned long)(GRIP_BUF)GRIP_Buf) >> 12) & 0xFFFF0uL) +
                 ((unsigned short)(GRIP_BUF)GRIP_Buf)
                + 0xF) >> 4 );

    memory = (GRIP_BUF_L)(((unsigned long)GRIP_DS) << 16);
}
{
  GRIP_DS := ((LongInt(GRIP_Buf) Shr 12) And $FFFF0) +
             ((LongInt(GRIP_Buf) + $F) Shr 4);
}
  GRIP_DS := (((LongInt(GRIP_Buf) Shr 12) And $FFFF0) +
             (LongInt(GRIP_Buf) And $FFFF) + $F) Shr 4;

  Memory  := ArPtr(LongInt(GRIP_DS) Shl 16);

  { Copy the image }

  For I := 0 To (Img_Size Shl 4) - 1 Do Memory^[I] := Core^[I];

  { Allocate a code selector for the core }

  GRIP_CS := GRIP_DS;

  { Prepare the thunking layer }

  For I := 0 To SizeOf(GRIP_THUNK) - 1 Do
   GRIP_Thunk[I] := Core^[I];

  GRIP_Thunk[2]  := GRIP_CS And $FF;
  GRIP_Thunk[3]  := GRIP_CS Shr 8;
  GRIP_Thunk[6]  := 0;
  GRIP_Thunk[7]  := 0;
  GRIP_Thunk[$C] := 0;
  GRIP_Thunk[$D] := 0;
  Memory^[$12]   := 0;
  Memory^[$13]   := 0;
  Memory^[$16]   := 0;
  Memory^[$17]   := 0;

  { Save the data selector }

  Memory^[$10] := GRIP_DS And $FF;
  Memory^[$11] := GRIP_DS Shr 8;
  Memory^[$14] := 0;
  Memory^[$15] := 0;

  { Complete the link }

  If Gr__Link = 0 Then
  Begin
    FreeMem(GRIP_Buf,GRIP_Size);
    GrLink := False;
    Exit;
  End;
  GRIP_Thunked := True;
  GrLink := True;
End; { GrLink }

Function GrUnlink: Boolean;
Begin

  { Check that the library has been loaded }

  If Not GRIP_Thunked Then
  Begin
    GrUnLink := False;
    Exit;
  End;

  { Unlink the library }
  Gr__Unlink;

  { Free the library memory }

  FreeMem(GRIP_Buf,GRIP_Size);

  GRIP_Thunked := False;

  GrUnlink := True;
End; { GrUnlink }

{$F+}
Procedure NewExitProc;
Begin
  ExitProc := OldExitProc;
  FreeMem(GrIP_Image,GrIP_ImgSize);
End; { NewExitProc }
{$F-}

Procedure LoadGrIP;
Var
  St   : String;
  F    : File;
  Size : LongInt;

  Function Exist(FileName: String): Boolean;
  Var Fil: File;
  Begin
    Assign(Fil,FileName);
    {$I-}
    Reset(Fil);
    {$I+}
    If (IOResult = 0) Then
    Begin
      Close(Fil);
      Exist := True;
    End
    Else Exist := False;
  End; { Exist }

Begin
  GrIP_Loaded := False;
  St := GetEnv(GrLibEnv);
  If St <> '' Then
  Begin
    GrLibDir := St;
    If GrLibDir[Length(GrLibDir)] <> '\' Then GrLibDir := GrLibDir + '\';
    If Exist(GrLibDir + GrLibName) Then
    Begin
      Assign(F,GrLibDir + GrLibName);
      Reset(F,1);
      Size := FileSize(F);
      If Size <= 32768 Then
      Begin
        GrIP_ImgSize := Size;
        GetMem(GrIP_Image,GrIP_ImgSize);
        If GrIP_Image <> Nil Then
        Begin
          BlockRead(F,GrIP_Image^,GrIP_ImgSize);
          GrIP_Loaded := True;
        End;
      End;
      Close(F);
    End;
  End;
End; { LoadGrIP }

Begin
  OldExitProc := ExitProc;
  ExitProc    := @NewExitProc;
  LoadGrIP;
End.

// ----------------------------------------------------------------------
// PCAE - PC Atari Emulator - Atari 2600 Emulator for MS-DOS
// Copyright (C) 2000 John Dullea
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
// ----------------------------------------------------------------------
